# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pkg/kast/interface/qt5/view/MainWindowView.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindowView(object):
    def setupUi(self, MainWindowView):
        MainWindowView.setObjectName("MainWindowView")
        MainWindowView.resize(800, 600)
        MainWindowView.setMinimumSize(QtCore.QSize(800, 600))
        self._widget = QtWidgets.QWidget(MainWindowView)
        self._widget.setObjectName("_widget")
        self._layout = QtWidgets.QVBoxLayout(self._widget)
        self._layout.setObjectName("_layout")
        self._centralLayout = QtWidgets.QVBoxLayout()
        self._centralLayout.setSpacing(0)
        self._centralLayout.setObjectName("_centralLayout")
        self.layoutForPreview = QtWidgets.QVBoxLayout()
        self.layoutForPreview.setSpacing(0)
        self.layoutForPreview.setObjectName("layoutForPreview")
        self._centralLayout.addLayout(self.layoutForPreview)
        self.layoutForControls = QtWidgets.QVBoxLayout()
        self.layoutForControls.setSpacing(0)
        self.layoutForControls.setObjectName("layoutForControls")
        self._centralLayout.addLayout(self.layoutForControls)
        self._centralLayout.setStretch(0, 1)
        self._layout.addLayout(self._centralLayout)
        MainWindowView.setCentralWidget(self._widget)
        self.menuBar = QtWidgets.QMenuBar(MainWindowView)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 800, 30))
        self.menuBar.setObjectName("menuBar")
        self.menuApplication = QtWidgets.QMenu(self.menuBar)
        self.menuApplication.setObjectName("menuApplication")
        MainWindowView.setMenuBar(self.menuBar)
        self.actionAbout = QtWidgets.QAction(MainWindowView)
        self.actionAbout.setObjectName("actionAbout")
        self.actionExit = QtWidgets.QAction(MainWindowView)
        self.actionExit.setObjectName("actionExit")
        self.menuApplication.addAction(self.actionAbout)
        self.menuApplication.addAction(self.actionExit)
        self.menuBar.addAction(self.menuApplication.menuAction())

        self.retranslateUi(MainWindowView)
        QtCore.QMetaObject.connectSlotsByName(MainWindowView)

    def retranslateUi(self, MainWindowView):
        _translate = QtCore.QCoreApplication.translate
        MainWindowView.setWindowTitle(_translate("MainWindowView", "MainWindowView"))
        self.menuApplication.setTitle(_translate("MainWindowView", "&Application"))
        self.actionAbout.setText(_translate("MainWindowView", "&About"))
        self.actionExit.setText(_translate("MainWindowView", "E&xit"))
