# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pkg/kast/interface/qt5/view/StatusBarView.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_StatusBarView(object):
    def setupUi(self, StatusBarView):
        StatusBarView.setObjectName("StatusBarView")
        StatusBarView.resize(660, 60)
        self.verticalLayout = QtWidgets.QVBoxLayout(StatusBarView)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self._separator = QtWidgets.QFrame(StatusBarView)
        self._separator.setFrameShape(QtWidgets.QFrame.HLine)
        self._separator.setFrameShadow(QtWidgets.QFrame.Sunken)
        self._separator.setObjectName("_separator")
        self.verticalLayout.addWidget(self._separator)
        self._layout = QtWidgets.QHBoxLayout()
        self._layout.setContentsMargins(-1, 3, -1, -1)
        self._layout.setSpacing(0)
        self._layout.setObjectName("_layout")
        self._layoutStatus = QtWidgets.QHBoxLayout()
        self._layoutStatus.setContentsMargins(-1, 6, -1, 6)
        self._layoutStatus.setSpacing(3)
        self._layoutStatus.setObjectName("_layoutStatus")
        self._labelStatus = QtWidgets.QLabel(StatusBarView)
        self._labelStatus.setObjectName("_labelStatus")
        self._layoutStatus.addWidget(self._labelStatus)
        self.labelStatusMessage = QtWidgets.QLabel(StatusBarView)
        self.labelStatusMessage.setObjectName("labelStatusMessage")
        self._layoutStatus.addWidget(self.labelStatusMessage)
        self._layoutStatus.setStretch(1, 1)
        self._layout.addLayout(self._layoutStatus)
        self.widgetProgress = QtWidgets.QWidget(StatusBarView)
        self.widgetProgress.setObjectName("widgetProgress")
        self._layoutProgress = QtWidgets.QHBoxLayout(self.widgetProgress)
        self._layoutProgress.setContentsMargins(0, 0, 0, 0)
        self._layoutProgress.setObjectName("_layoutProgress")
        self.progressBar = QtWidgets.QProgressBar(self.widgetProgress)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setInvertedAppearance(False)
        self.progressBar.setObjectName("progressBar")
        self._layoutProgress.addWidget(self.progressBar)
        self.buttonCancel = QtWidgets.QPushButton(self.widgetProgress)
        self.buttonCancel.setText("")
        self.buttonCancel.setObjectName("buttonCancel")
        self._layoutProgress.addWidget(self.buttonCancel)
        self._layoutProgress.setStretch(0, 1)
        self._layout.addWidget(self.widgetProgress)
        self.verticalLayout.addLayout(self._layout)

        self.retranslateUi(StatusBarView)
        QtCore.QMetaObject.connectSlotsByName(StatusBarView)

    def retranslateUi(self, StatusBarView):
        _translate = QtCore.QCoreApplication.translate
        StatusBarView.setWindowTitle(_translate("StatusBarView", "StatusBarView"))
        self._labelStatus.setText(_translate("StatusBarView", "Status:"))
        self.labelStatusMessage.setText(_translate("StatusBarView", "Idle"))
        self.buttonCancel.setToolTip(_translate("StatusBarView", "Cancel"))
