# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pkg/kast/interface/qt5/view/VideoSettingsView.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VideoSettingsView(object):
    def setupUi(self, VideoSettingsView):
        VideoSettingsView.setObjectName("VideoSettingsView")
        VideoSettingsView.resize(604, 173)
        VideoSettingsView.setWindowTitle("VideoSettingsView")
        self._layout = QtWidgets.QVBoxLayout(VideoSettingsView)
        self._layout.setObjectName("_layout")
        self._layoutMain = QtWidgets.QWidget(VideoSettingsView)
        self._layoutMain.setObjectName("_layoutMain")
        self.verticalLayout = QtWidgets.QVBoxLayout(self._layoutMain)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self._layoutDeviceAndVideo = QtWidgets.QGridLayout()
        self._layoutDeviceAndVideo.setContentsMargins(0, 0, 0, 10)
        self._layoutDeviceAndVideo.setHorizontalSpacing(10)
        self._layoutDeviceAndVideo.setVerticalSpacing(6)
        self._layoutDeviceAndVideo.setObjectName("_layoutDeviceAndVideo")
        self.lineEditVideo = QtWidgets.QLineEdit(self._layoutMain)
        self.lineEditVideo.setEnabled(True)
        self.lineEditVideo.setReadOnly(True)
        self.lineEditVideo.setObjectName("lineEditVideo")
        self._layoutDeviceAndVideo.addWidget(self.lineEditVideo, 1, 1, 1, 1)
        self._labelAudio = QtWidgets.QLabel(self._layoutMain)
        self._labelAudio.setObjectName("_labelAudio")
        self._layoutDeviceAndVideo.addWidget(self._labelAudio, 2, 0, 1, 1)
        self._labelDevice = QtWidgets.QLabel(self._layoutMain)
        self._labelDevice.setObjectName("_labelDevice")
        self._layoutDeviceAndVideo.addWidget(self._labelDevice, 0, 0, 1, 1)
        self.buttonSubtitlesAdd = QtWidgets.QPushButton(self._layoutMain)
        self.buttonSubtitlesAdd.setText("")
        self.buttonSubtitlesAdd.setObjectName("buttonSubtitlesAdd")
        self._layoutDeviceAndVideo.addWidget(self.buttonSubtitlesAdd, 2, 3, 1, 1)
        self._layoutAudioAndSubtitles = QtWidgets.QGridLayout()
        self._layoutAudioAndSubtitles.setContentsMargins(0, 0, 0, 0)
        self._layoutAudioAndSubtitles.setHorizontalSpacing(10)
        self._layoutAudioAndSubtitles.setVerticalSpacing(0)
        self._layoutAudioAndSubtitles.setObjectName("_layoutAudioAndSubtitles")
        self._labelSubtitles = QtWidgets.QLabel(self._layoutMain)
        self._labelSubtitles.setObjectName("_labelSubtitles")
        self._layoutAudioAndSubtitles.addWidget(self._labelSubtitles, 0, 2, 1, 1)
        self.comboBoxAudio = QtWidgets.QComboBox(self._layoutMain)
        self.comboBoxAudio.setObjectName("comboBoxAudio")
        self._layoutAudioAndSubtitles.addWidget(self.comboBoxAudio, 0, 1, 1, 1)
        self.comboBoxSubtitles = QtWidgets.QComboBox(self._layoutMain)
        self.comboBoxSubtitles.setObjectName("comboBoxSubtitles")
        self._layoutAudioAndSubtitles.addWidget(self.comboBoxSubtitles, 0, 3, 1, 1)
        self._layoutAudioAndSubtitles.setColumnStretch(1, 1)
        self._layoutAudioAndSubtitles.setColumnStretch(3, 1)
        self._layoutDeviceAndVideo.addLayout(self._layoutAudioAndSubtitles, 2, 1, 1, 1)
        self.comboBoxDevice = QtWidgets.QComboBox(self._layoutMain)
        self.comboBoxDevice.setObjectName("comboBoxDevice")
        self._layoutDeviceAndVideo.addWidget(self.comboBoxDevice, 0, 1, 1, 1)
        self.buttonDeviceRefresh = QtWidgets.QPushButton(self._layoutMain)
        self.buttonDeviceRefresh.setText("")
        self.buttonDeviceRefresh.setObjectName("buttonDeviceRefresh")
        self._layoutDeviceAndVideo.addWidget(self.buttonDeviceRefresh, 0, 3, 1, 1)
        self._labelVideo = QtWidgets.QLabel(self._layoutMain)
        self._labelVideo.setObjectName("_labelVideo")
        self._layoutDeviceAndVideo.addWidget(self._labelVideo, 1, 0, 1, 1)
        self.buttonVideoOpen = QtWidgets.QPushButton(self._layoutMain)
        self.buttonVideoOpen.setText("")
        self.buttonVideoOpen.setObjectName("buttonVideoOpen")
        self._layoutDeviceAndVideo.addWidget(self.buttonVideoOpen, 1, 3, 1, 1)
        self.verticalLayout.addLayout(self._layoutDeviceAndVideo)
        self._layoutStream = QtWidgets.QHBoxLayout()
        self._layoutStream.setObjectName("_layoutStream")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._layoutStream.addItem(spacerItem)
        self.buttonStream = QtWidgets.QPushButton(self._layoutMain)
        self.buttonStream.setObjectName("buttonStream")
        self._layoutStream.addWidget(self.buttonStream)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._layoutStream.addItem(spacerItem1)
        self.verticalLayout.addLayout(self._layoutStream)
        self._layout.addWidget(self._layoutMain)

        self.retranslateUi(VideoSettingsView)
        QtCore.QMetaObject.connectSlotsByName(VideoSettingsView)

    def retranslateUi(self, VideoSettingsView):
        _translate = QtCore.QCoreApplication.translate
        self.lineEditVideo.setToolTip(_translate("VideoSettingsView", "Video file path"))
        self._labelAudio.setText(_translate("VideoSettingsView", "Audio:"))
        self._labelDevice.setText(_translate("VideoSettingsView", "Device:"))
        self.buttonSubtitlesAdd.setToolTip(_translate("VideoSettingsView", "Open subtitles"))
        self._labelSubtitles.setText(_translate("VideoSettingsView", "Subtitles:"))
        self.comboBoxAudio.setToolTip(_translate("VideoSettingsView", "Audio tracks"))
        self.comboBoxSubtitles.setToolTip(_translate("VideoSettingsView", "Subtitles"))
        self.comboBoxDevice.setToolTip(_translate("VideoSettingsView", "Cast devices"))
        self.buttonDeviceRefresh.setToolTip(_translate("VideoSettingsView", "Search for cast devices"))
        self._labelVideo.setText(_translate("VideoSettingsView", "Video:"))
        self.buttonVideoOpen.setToolTip(_translate("VideoSettingsView", "Open video"))
        self.buttonStream.setToolTip(_translate("VideoSettingsView", "Start streaming"))
        self.buttonStream.setText(_translate("VideoSettingsView", "<< Stream >>"))
