# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pkg/kast/interface/qt5/view/AboutViewModel.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutViewModel(object):
    def setupUi(self, AboutViewModel):
        AboutViewModel.setObjectName("AboutViewModel")
        AboutViewModel.setWindowModality(QtCore.Qt.NonModal)
        AboutViewModel.resize(136, 170)
        AboutViewModel.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        AboutViewModel.setSizeGripEnabled(False)
        AboutViewModel.setModal(True)
        self._layout = QtWidgets.QVBoxLayout(AboutViewModel)
        self._layout.setContentsMargins(6, 6, 6, 6)
        self._layout.setObjectName("_layout")
        self._layoutContent = QtWidgets.QHBoxLayout()
        self._layoutContent.setObjectName("_layoutContent")
        self.labelLogo = QtWidgets.QLabel(AboutViewModel)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelLogo.sizePolicy().hasHeightForWidth())
        self.labelLogo.setSizePolicy(sizePolicy)
        self.labelLogo.setText("")
        self.labelLogo.setObjectName("labelLogo")
        self._layoutContent.addWidget(self.labelLogo)
        self._layoutLabels = QtWidgets.QVBoxLayout()
        self._layoutLabels.setObjectName("_layoutLabels")
        self._labelAppName = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAppName.setFont(font)
        self._labelAppName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAppName.setObjectName("_labelAppName")
        self._layoutLabels.addWidget(self._labelAppName)
        self._labelAppVersion = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAppVersion.setFont(font)
        self._labelAppVersion.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAppVersion.setObjectName("_labelAppVersion")
        self._layoutLabels.addWidget(self._labelAppVersion)
        self._labelAppDescription = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAppDescription.setFont(font)
        self._labelAppDescription.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAppDescription.setObjectName("_labelAppDescription")
        self._layoutLabels.addWidget(self._labelAppDescription)
        self._labelAuthorName = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAuthorName.setFont(font)
        self._labelAuthorName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAuthorName.setObjectName("_labelAuthorName")
        self._layoutLabels.addWidget(self._labelAuthorName)
        self._labelAuthorContact = QtWidgets.QLabel(AboutViewModel)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self._labelAuthorContact.setFont(font)
        self._labelAuthorContact.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self._labelAuthorContact.setObjectName("_labelAuthorContact")
        self._layoutLabels.addWidget(self._labelAuthorContact)
        self._layoutContent.addLayout(self._layoutLabels)
        self._spacer = QtWidgets.QFrame(AboutViewModel)
        self._spacer.setFrameShape(QtWidgets.QFrame.VLine)
        self._spacer.setFrameShadow(QtWidgets.QFrame.Sunken)
        self._spacer.setObjectName("_spacer")
        self._layoutContent.addWidget(self._spacer)
        self._layoutDetails = QtWidgets.QVBoxLayout()
        self._layoutDetails.setObjectName("_layoutDetails")
        self.labelAppName = QtWidgets.QLabel(AboutViewModel)
        self.labelAppName.setObjectName("labelAppName")
        self._layoutDetails.addWidget(self.labelAppName)
        self.labelAppVersion = QtWidgets.QLabel(AboutViewModel)
        self.labelAppVersion.setObjectName("labelAppVersion")
        self._layoutDetails.addWidget(self.labelAppVersion)
        self.labelAppDescription = QtWidgets.QLabel(AboutViewModel)
        self.labelAppDescription.setObjectName("labelAppDescription")
        self._layoutDetails.addWidget(self.labelAppDescription)
        self.labelAuthorName = QtWidgets.QLabel(AboutViewModel)
        self.labelAuthorName.setObjectName("labelAuthorName")
        self._layoutDetails.addWidget(self.labelAuthorName)
        self.labelAuthorContact = QtWidgets.QLabel(AboutViewModel)
        self.labelAuthorContact.setObjectName("labelAuthorContact")
        self._layoutDetails.addWidget(self.labelAuthorContact)
        self._layoutContent.addLayout(self._layoutDetails)
        self._layoutContent.setStretch(3, 1)
        self._layout.addLayout(self._layoutContent)
        self._buttonBox = QtWidgets.QDialogButtonBox(AboutViewModel)
        self._buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self._buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self._buttonBox.setObjectName("_buttonBox")
        self._layout.addWidget(self._buttonBox)

        self.retranslateUi(AboutViewModel)
        self._buttonBox.accepted.connect(AboutViewModel.accept)
        self._buttonBox.rejected.connect(AboutViewModel.reject)
        QtCore.QMetaObject.connectSlotsByName(AboutViewModel)

    def retranslateUi(self, AboutViewModel):
        _translate = QtCore.QCoreApplication.translate
        AboutViewModel.setWindowTitle(_translate("AboutViewModel", "About"))
        self._labelAppName.setText(_translate("AboutViewModel", "Application:"))
        self._labelAppVersion.setText(_translate("AboutViewModel", "Version:"))
        self._labelAppDescription.setText(_translate("AboutViewModel", "Description:"))
        self._labelAuthorName.setText(_translate("AboutViewModel", "Author:"))
        self._labelAuthorContact.setText(_translate("AboutViewModel", "Contact:"))
        self.labelAppName.setText(_translate("AboutViewModel", "..."))
        self.labelAppVersion.setText(_translate("AboutViewModel", "..."))
        self.labelAppDescription.setText(_translate("AboutViewModel", "..."))
        self.labelAuthorName.setText(_translate("AboutViewModel", "..."))
        self.labelAuthorContact.setText(_translate("AboutViewModel", "..."))
